const HN_POLL_ITEM = '30210378';
const resultsPlaceholder = document.getElementById('results-placeholder');
const resultsTable = document.getElementById('results-table');
const resultsTableBody = document.querySelector('#results-table tbody');

async function getHNItem(itemNumber) {
  // Uses the Hacker News API: https://github.com/HackerNews/API
  const response = await fetch(`https://hacker-news.firebaseio.com/v0/item/${itemNumber}.json`);
  return response.json();
}

(async function () {
  try {
    const poll = await getHNItem(HN_POLL_ITEM);
    const promisedResults = poll.parts.map(async (pollChoice) => {
      const result = await getHNItem(pollChoice);
      // Each poll option on HN starts with a score of 1, so subtract that.
      result.votes = result.score - 1;
      return result;
    });
    const results = await Promise.all(promisedResults);
    const totalVotes = results.reduce((sum, item) => sum + item.votes, 0);

    // Add a row that shows the calculated total for the United States.
    const unitedStatesTotalResult = results.reduce((accumulator, item) => {
      if (item.text.indexOf('US:') === 0) {
        accumulator.votes += item.votes;
      }
      return accumulator;
    }, { text: 'US total <i>(calculated)</i>', votes: 0 });
    results.push(unitedStatesTotalResult);

    // Calculate each option's percentage of the total votes.
    results.forEach(result => {
      const percentage = 100 * result.votes / totalVotes;
      const percentageString = percentage.toFixed(1);
      if (percentageString === '0.0' && result.votes > 0) {
        // Use 2 decimal places for countries that received low votes.
        result.percentage = percentage.toFixed(2);
      } else {
        // Omit the trailing .0 to make it look nicer.
        result.percentage = percentageString.split('.0')[0];
      }
    });

    results.sort((a, b) => {
      if (a.votes == b.votes) {
        // Sort alphabetically.
        return (a.text > b.text) ? 1 : -1
      }
      // Sort by votes.
      return (a.votes > b.votes) ? -1 : 1;
    });

    // Append the total count to the end.
    results.push({ text: '<b>Total</b>', votes: `<b>${totalVotes}</b>`, percentage: '100' });

    const tableRows = results.map(result => `<tr><td>${result.text}</td><td align="center">${result.votes}</td><td align="center">${result.percentage}%</td></tr>`);
    resultsTableBody.innerHTML = tableRows.join('');
    resultsTable.style.display = 'table';
    resultsPlaceholder.style.display = 'none';
  } catch (error) {
    console.error(error);
    resultsPlaceholder.innerText = 'An error occurred: ' + error;
  }
})();
